'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SuiTealiumTracker = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _justDebounce = require('just-debounce');

var _justDebounce2 = _interopRequireDefault(_justDebounce);

var _events = require('@s-ui/js/lib/events');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ELEMENTS_CLICKABLE = ['A', 'BUTTON', 'DIV', 'INPUT', 'SVG'];
var ELEMENT_NODE = 1;
var THROTTLE_WAIT = 100;

/**
 * SuiTealiumTracker
 */

var SuiTealiumTracker = exports.SuiTealiumTracker = function () {
  function SuiTealiumTracker() {
    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        customEventName = _ref.customEventName,
        handleAnchorDelay = _ref.handleAnchorDelay;

    (0, _classCallCheck3.default)(this, SuiTealiumTracker);

    this.customEventName = customEventName;
    this.handleAnchorDelay = handleAnchorDelay;
    this.sendTealiumThrottled = (0, _justDebounce2.default)(this.sendTealium, THROTTLE_WAIT, true);
  }

  (0, _createClass3.default)(SuiTealiumTracker, [{
    key: 'init',
    value: function init() {
      this.initClickListener();
      this.initChangeListener();
      this.customEventName && this.promoteDispatchCustomEventToWindow();
      this.customEventName && this.initCustomEventListener();
    }
  }, {
    key: 'initClickListener',
    value: function initClickListener() {
      var _this = this;

      document.addEventListener('click', function (e) {
        var target = e.target;
        var node = target;
        do {
          if (node.nodeType !== ELEMENT_NODE) {
            break;
          }
          _this.checkIfNeedToTrackClick({ node: node, e: e });
          var parentTagName = node.parentElement && node.parentElement.tagName.toUpperCase();
          // search if the parent is a clickable element to check, otherwise we stop
          node = ELEMENTS_CLICKABLE.includes(parentTagName) ? node.parentElement : false;
        } while (node);
      });
    }
  }, {
    key: 'initChangeListener',
    value: function initChangeListener() {
      var _this2 = this;

      document.addEventListener('change', function (_ref2) {
        var target = _ref2.target;
        var _target$options = target.options,
            options = _target$options === undefined ? [] : _target$options,
            _target$selectedIndex = target.selectedIndex,
            selectedIndex = _target$selectedIndex === undefined ? 0 : _target$selectedIndex;

        var option = options[selectedIndex] || { dataset: {} };
        var tealiumTag = option.dataset.tealiumTag;

        if (tealiumTag) {
          _this2.sendTealiumThrottled(tealiumTag);
        }
      });
    }
  }, {
    key: 'initCustomEventListener',
    value: function initCustomEventListener() {
      var _this3 = this;

      document.addEventListener(this.customEventName, function (evt) {
        var tealiumTag = evt.detail.tealiumTag;

        _this3.sendTealiumThrottled(tealiumTag);
      });
    }
  }, {
    key: 'checkIfNeedToTrackClick',
    value: function checkIfNeedToTrackClick(_ref3) {
      var node = _ref3.node,
          e = _ref3.e;
      var _node$dataset = node.dataset,
          dataset = _node$dataset === undefined ? {} : _node$dataset;
      var tealiumTag = dataset.tealiumTag;

      var args = [dataset];
      tealiumTag && node.tagName === 'A' && this.handleAnchorDelay && args.push(this.delayAnchorLinkCall(e));
      tealiumTag && this.sendTealiumThrottled.apply(this, args);
    }
  }, {
    key: 'delayAnchorLinkCall',
    value: function delayAnchorLinkCall(e) {
      e.preventDefault();
      return function () {
        setTimeout(function () {
          return window.location.replace(e.target.href);
        }, 300);
      };
    }
  }, {
    key: 'promoteDispatchCustomEventToWindow',
    value: function promoteDispatchCustomEventToWindow() {
      var _this4 = this;

      window.dispatchCustomEvent = function (detail) {
        return (0, _events.dispatchEvent)({ eventName: _this4.customEventName, detail: detail });
      };
    }
  }, {
    key: 'sendTealium',
    value: function sendTealium(data, callback) {
      var args = [(0, _extends3.default)({}, window.utag_data, data)];
      callback && args.push(callback);
      window.utag ? window.utag.link.apply(window.utag, args) : console.warn('There is no utag present on this site no event was dispatched.');
    }
  }]);
  return SuiTealiumTracker;
}();